@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2017 J.F.Dockes
+# Copyright (C) 2017-2020 J.F.Dockes
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,25 +23,34 @@
 # Object Ids inside the subsections: complicated
 
 import sys
-PY3 = sys.version > '3'
 import os
 import sqlite3
 import time
 import tempfile
+import time
 
-from upmplgutils import uplog
-from uprclutils import g_myprefix, rcldirentry, rcldoctoentry, cmpentries
+from upmplgutils import uplog, direntry
+import uprclutils
+from uprclutils import rcldoctoentry, cmpentries
 import uprclinit
-from uprcltagscreate import recolltosql, _clid, g_tagtotable, \
-     g_tagdisplaytag, g_indextags
+import uprcltagscreate
+from uprcltagscreate import _clid, recolltosql
 
 # The browseable object which defines the tree of tracks organized by tags.
 class Tagged(object):
+    # We maintain a cache for the total count of query statements
+    # which are expensive to compute when we are returning a partial
+    # slice (which we currently only do when displaying the top-level
+    # items list, so the cache can actually never have more than 1 element...)
+    _stmt_cnt_cachesize = 20
     def __init__(self, rcldocs, httphp, pathprefix):
         self._httphp = httphp
         self._pprefix = pathprefix
         self._conn = None
         self._init_sqconn()
+        self._stmt_cnt_cache = {}
+        self._stmt_cnt_cachequeue = []
+        self.hidden = []
         recolltosql(self._conn, rcldocs)
         
 
@@ -66,7 +75,7 @@ class Tagged(object):
         uplog("rootentries: pid %s path %s" % (pid, path))
         entries = []
         nalbs = self._albcntforfolder(path)
-        entries.append(rcldirentry(pid + 'albums', pid, nalbs + ' albums'))
+        entries.append(direntry(pid + 'albums', pid, nalbs + ' albums'))
         if path:
             where = ' WHERE tracks.path LIKE ? '
             args = (path + '%',)
@@ -77,11 +86,11 @@ class Tagged(object):
         stmt = "SELECT COUNT(*) from tracks"
         c.execute(stmt+where, args)
         nitems = str(c.fetchone()[0])
-        entries.append(rcldirentry(pid + 'items', pid, nitems + ' items'))
+        entries.append(direntry(pid + 'items', pid, nitems + ' items'))
         subqs = self._subtreetags(where, args)
+        tagdisplaytag = uprcltagscreate.getTagDisplayTag()
         for tt in subqs:
-            entries.append(rcldirentry(pid + '=' + tt , pid,
-                                       g_tagdisplaytag[tt]))
+            entries.append(direntry(pid + '=' + tt , pid, tagdisplaytag[tt]))
         return entries
 
 
@@ -89,8 +98,10 @@ class Tagged(object):
     def _subtreetags(self, where, values):
         c = self._conn.cursor()
         tags = []
-        for tt in g_indextags:
-            tb = g_tagtotable[tt]
+        indextags = uprcltagscreate.getIndexTags()
+        tagtotable = uprcltagscreate.getTagToTable()
+        for tt in indextags:
+            tb = tagtotable[tt]
             stmt = '''SELECT COUNT(DISTINCT %s) FROM tracks %s''' % \
                    (_clid(tb), where)
             #uplog("subtreetags: stmt: [%s]" % stmt)
@@ -103,18 +114,58 @@ class Tagged(object):
                 tags.append(tt)
         return tags
 
-
+    def _stmt_total(self, stmt, values):
+        try:
+            total = self._stmt_cnt_cache[stmt]
+        except:
+            c = self._conn.cursor()
+            c.execute(stmt, values)
+            ids = [r[0] for r in c]
+            total = len(ids)
+            self._stmt_cnt_cache[stmt] = total
+            self._stmt_cnt_cachequeue.append(stmt)
+            if len(self._stmt_cnt_cachequeue) > self._stmt_cnt_cachesize:
+                del(self._stmt_cnt_cache[self._stmt_cnt_cachequeue[0]])
+                self._stmt_cnt_cachequeue = self._stmt_cnt_cachequeue[1:]
+        return total
+        
     # Build a list of track directory entries for an SQL statement
     # which selects docidxs (SELECT docidx,... FROM tracks WHERE...)
-    def _trackentriesforstmt(self, stmt, values, pid):
-        rcldocs = uprclinit.g_trees['folders'].rcldocs()
+    #
+    # NOTE: setting offset or cnt currently precludes sorting, which
+    # is unavoidable if we don't cache the whole result (which would
+    # use a lot of memory for big collections, around 100MB for 10k
+    # songs, significant on a small SBC). At the moment, we only do
+    # this for the full top-level items list, which is not really
+    # useful anyway.
+    def _trackentriesforstmt(self, stmt, values, pid, offset=0, count=0):
+        uplog("trackentries: offset %d count %d" % (offset, count))
+        total = 0
+        if offset != 0 or count != 0:
+            total = self._stmt_total(stmt, values)
+        if total < 1000:
+            # Then return the full set: this is acceptable in terms of
+            # performances, and will allow the result to be sorted.
+            # plgwithslave knows how to deal with a full set returned
+            # from a partial request.
+            count = 0
+            offset = 0
+        if count != 0:
+            stmt += " LIMIT %d " % count
+        if offset != 0:
+            stmt += " OFFSET %d " % offset
+        rcldocs = uprclinit.getTree('folders').rcldocs()
         c = self._conn.cursor()
         c.execute(stmt, values)
         entries = [rcldoctoentry(pid + '$i' + str(r[0]),
                                  pid, self._httphp, self._pprefix,
                                  rcldocs[r[0]]) for r in c]
-        return sorted(entries, key=cmpentries)
-    
+        uplog("trackentries: stmt returns %d entries" % len(entries))
+        if offset != 0 or count != 0:
+            return (offset, total, entries)
+        else:
+            return sorted(entries, key=cmpentries)
+
 
     # Return a list of trackids as selected by the current
     # path <selwhere> is like: WHERE col1_id = ? AND col2_id = ? [...], and
@@ -261,10 +312,27 @@ class Tagged(object):
         for r in c:
             id = pid + '$' + str(r[0])
             entries.append(
-                rcldirentry(id, pid, r[1], arturi=r[2], date=r[3],artist=r[4],
+                direntry(id, pid, r[1], arturi=r[2], date=r[3],artist=r[4],
                             upnpclass='object.container.album.musicAlbum'))
         return entries
 
+    # Called when the search finds one of our synthetic album search
+    # results. Create a container entry for it
+    def direntryforalbid(self, albid):
+        intalbid = int(albid)
+        c = self._conn.cursor()
+        stmt = '''SELECT album_id, albtitle, albarturi, albdate, artist.value
+        FROM albums LEFT JOIN artist ON artist.artist_id = albums.artist_id
+        WHERE album_id = ? AND albtdisc is NULL  ORDER BY albtitle'''
+        args = (intalbid,)
+        c.execute(stmt, args)
+        for r in c:
+            pid = uprclinit.getObjPrefix() + 'albums'
+            id = pid + '$' + albid
+            return direntry(id, pid, r[1], arturi=r[2], date=r[3],artist=r[4],
+                               upnpclass='object.container.album.musicAlbum')
+        return None
+
 
     # This is called when an 'albums' element is encountered in the
     # selection path. i is the index of the albums element. The tree under
@@ -300,7 +368,7 @@ class Tagged(object):
             entries = self._trackentriesforstmt(stmt, rawalbids+docidsl, pid)
             if ntracks != len(entries):
                 id = pid + '$' + 'showca'
-                entries = [rcldirentry(id, pid, '>> Complete Album')] + entries
+                entries = [direntry(id, pid, '>> Complete Album')] + entries
         elif i == len(qpath)-3:
             # 'Complete album' entry
             # Note that minim has an additional level here, probably to
@@ -335,14 +403,14 @@ class Tagged(object):
             # Replace $items with $albums for the album entry
             id = pid.replace('$items', '$albums') + '$' + str(albid) + '$showca'
             if len(tlist) != len(docids):
-                entries.append(rcldirentry(id, pid, '>> Complete Album'))
+                entries.append(direntry(id, pid, '>> Complete Album'))
             else:
                 displaytracks = False
                 el = self._direntriesforalbums(pid, "WHERE album_id = %s"%albid)
                 el[0]['id'] = id
                 entries.append(el[0])
         if displaytracks:
-            rcldocs = uprclinit.g_trees['folders'].rcldocs()
+            rcldocs = uprclinit.getTree('folders').rcldocs()
             entries += [rcldoctoentry(pid + '$i' + str(docid),
                                       pid, self._httphp, self._pprefix,
                                       rcldocs[docid]) for docid in docids]
@@ -355,6 +423,8 @@ class Tagged(object):
     def _tagsbrowse(self, pid, qpath, flag, path=''):
         uplog("tagsbrowse. pid %s qpath %s" % (pid, qpath))
 
+        tagdisplaytag = uprcltagscreate.getTagDisplayTag()
+
         # Walk the qpath, which was generated from the objid and
         # defines what tracks are selected and what we want to
         # display. E.g =Artist$21$=Date would display all distinct
@@ -368,6 +438,7 @@ class Tagged(object):
         else:
             selwhere = ''
             values = []
+        tagtotable = uprcltagscreate.getTagToTable()
         i = 0
         while i < qlen:
             elt = qpath[i]
@@ -384,7 +455,7 @@ class Tagged(object):
             # in different ways depending if this is the last element or
             # not.
             if elt.startswith('='):
-                col = g_tagtotable[elt[1:]] 
+                col = tagtotable[elt[1:]] 
 
             selwhere = selwhere + ' AND ' if selwhere else ' WHERE '
             if i == qlen - 1:
@@ -416,7 +487,7 @@ class Tagged(object):
             if len(albids) > 1:
                 id = pid + '$albums'
                 label = '%d albums'
-                entries.append(rcldirentry(id, pid, label % len(albids)))
+                entries.append(direntry(id, pid, label % len(albids)))
             elif len(albids) == 1 and not subqs:
                 # Only display '>> Complete album' if not all tracks
                 # already there. If all tracks are there, we display
@@ -425,7 +496,7 @@ class Tagged(object):
                 tlist = self._trackentriesforalbum(albid, pid)
                 id = pid + '$albums$' + str(albid) + '$showca'
                 if len(tlist) != len(docids):
-                    entries.append(rcldirentry(id, pid, '>> Complete Album'))
+                    entries.append(direntry(id, pid, '>> Complete Album'))
                 else:
                     displaytracks = False
                     el = self._direntriesforalbums(pid,
@@ -436,12 +507,12 @@ class Tagged(object):
             if subqs:
                 id = pid + '$items'
                 label = '%d items'
-                entries.append(rcldirentry(id, pid, label % len(docids)))
+                entries.append(direntry(id, pid, label % len(docids)))
                 for tt in subqs:
                     id = pid + '$=' + tt
-                    entries.append(rcldirentry(id, pid, g_tagdisplaytag[tt]))
+                    entries.append(direntry(id, pid, tagdisplaytag[tt]))
             elif displaytracks:
-                rcldocs = uprclinit.g_trees['folders'].rcldocs()
+                rcldocs = uprclinit.getTree('folders').rcldocs()
                 for docidx in docids:
                     id = pid + '$*i' + str(docidx)
                     entries.append(rcldoctoentry(id, pid, self._httphp,
@@ -462,7 +533,7 @@ class Tagged(object):
             c.execute(stmt, values)
             for r in c:
                 id = pid + '$' + str(r[0])
-                entries.append(rcldirentry(id, pid, r[1]))
+                entries.append(direntry(id, pid, r[1]))
         return entries
 
 
@@ -485,13 +556,13 @@ class Tagged(object):
 
 
     # Implement the common part of browse() and browseFolder()
-    def _dobrowse(self, pid, flag, qpath, folder=''):
-        uplog("Tags:browsFolder: qpath %s"%qpath)
+    def _dobrowse(self, pid, flag, qpath, folder='', offset=0, count=0):
+        uplog("Tags:_dobrowse: qpath %s"%qpath)
         if qpath[0] == 'items':
             args = (folder + '%',) if folder else ()
             folderwhere = ' WHERE tracks.path LIKE ? ' if folder else ' '
             stmt = 'SELECT docidx FROM tracks' + folderwhere
-            entries = self._trackentriesforstmt(stmt, args, pid)
+            entries = self._trackentriesforstmt(stmt, args, pid, offset, count)
         elif qpath[0] == 'albums':
             entries = self._albumsbrowse(pid, qpath, flag, folder)
         elif qpath[0].startswith('='):
@@ -501,27 +572,59 @@ class Tagged(object):
         return entries
         
 
-    # Call from the folders tree when Tag View is selected.
+    # Call from the folders tree when Tag View is selected. Reproduces
+    # the general view by tags, but with selection for only the files
+    # under the specified folder.
+    #
+    # Somewhat like minim: under ">> Tag View", 2 entries, ">> Hide
+    # Contents/" and simplename/ (last elt of folder path). If ">> Hide
+    # Contents/" is visited, like would happen with a CP recursive
+    # walk, simplename/ is marked for 2 S and will be empty if
+    # visited. If ">> Hide Contents/" is not visited, or after 2 S,
+    # simplename/ will contain the local tags subtree.
+    #
     def browseFolder(self, pid, flag, pthremain, folder):
         uplog("Tags:browseFolder: pid %s pth %s fld %s"%(pid,pthremain,folder))
         l = pthremain.split('$')
         # 1st elt in list is empty because pthremain begins with $. so
         # len(l)==2 is the root of tags from this folder
+        entries = []
+        if len(l) < 2:
+            # ??
+            return entries
         if len(l) == 2:
-            return self.rootentries(pid + '$', folder)
+            entries.append(direntry(pid + '$hchide', pid, ">> Hide Contents"))
+            entries.append(direntry(pid + '$hctags', pid, uprclutils.basename(folder)))
+            return entries
+        elif len(l) == 3:
+            ppid = '$'.join(pid.split('$')[:-1])
+            value = l[-1]
+            if value == "hchide":
+                self.hidden.insert(0, (time.time(), ppid))
+                return entries
+            else:
+                now = time.time()
+                for i in range(len(self.hidden)):
+                    #uplog("Browsefolder: hidden: since %d pid %s" %
+                    #      (int(now - self.hidden[i][0]), self.hidden[i][1]))
+                    if self.hidden[i][0] <= now - 2:
+                        self.hidden = self.hidden[:i]
+                        break
+                    elif self.hidden[i][1] == ppid:
+                        return entries
+                return self.rootentries(pid + '$', folder)
         else:
-            qpath = l[2:]
+            qpath = l[3:]
             return self._dobrowse(pid, flag, qpath, folder)
 
         
     # Top level browse routine. Handle the special cases and call the
     # appropriate worker routine.
-    def browse(self, pid, flag):
-        idpath = pid.replace(g_myprefix, '', 1)
+    def browse(self, pid, flag, offset, count):
+        idpath = pid.replace(uprclinit.getObjPrefix(), '', 1)
         uplog('tags:browse: idpath <%s>' % idpath)
-        entries = []
         qpath = idpath.split('$')
-        return self._dobrowse(pid, flag, qpath)
+        return self._dobrowse(pid, flag, qpath, offset=offset, count=count)
 
 
 
